@@ -23,12 +23,10 @@ DEFAULT_FEATURE_NAME = ["index", "_index", "Gene", "accession_id"]
 DEFAULT_DIMRED_KEYS = {"coors":["X", "Y"], "tsne":["_tSNE_1", "_tSNE_2"]}
 DEFAULT_ABLOOM_BARCODE_NAME = "observation_id"
 DEFAULT_ABLOOM_FEATURE_NAME = "accession_id"
-SCANPYDATA_DEFAULT_GRAPH_BASED = "louvain"
-SPRINGDATA_DEFAULT_GRAPH_BASED = "ClustersWT"
-LOOMDATA_DEFAULT_GRAPH_BASED = "ClusterID"
 
 class DataObject(ABC):
-    def __init__(self, source, graph_based, replace_missing="Unassigned",
+    def __init__(self, source, graph_based = None, replace_missing="Unassigned",
+                    sharing_history = None,
                     **kwargs):
         """Constructor of DataObject class
 
@@ -66,6 +64,15 @@ class DataObject(ABC):
                                     % self.unit,
                                     allowed_units)
 
+        if sharing_history is None:
+            self.sharing_history = []
+        else:
+            self.sharing_history = sharing_history.copy()
+
+        history_obj = generate_history_object()
+        history_obj["hash_id"] = self.study_id
+        self.sharing_history.append(history_obj)
+
     def __del__(self):
         self.close()
 
@@ -623,8 +630,6 @@ class DataObject(ABC):
         return has_raw
 
     def create_runinfo(self):
-        runinfo_history = generate_history_object()
-        runinfo_history["hash_id"] = self.study_id
         date = time.time() * 1000
         run_info = {
             "species":self.species,
@@ -639,7 +644,7 @@ class DataObject(ABC):
             "platform":"unknown",
             "omics":self.omics,
             "title":self.title,
-            "history":[runinfo_history],
+            "history":self.sharing_history,
             "unit":self.unit
         }
         misc = self.get_misc()
@@ -746,7 +751,7 @@ class DataObject(ABC):
         return self.study_id
 
 class ScanpyData(DataObject):
-    def __init__(self, source, graph_based, raw_key="counts", pca_key="X_pca",
+    def __init__(self, source, graph_based = None, raw_key="counts", pca_key="X_pca",
                     cite_seq_suffix=None,
                     **kwargs):
         """Constructor of ScanpyData object
@@ -756,14 +761,18 @@ class ScanpyData(DataObject):
             graph_based: Name of metadata that is the result of clustering process
             raw_key: Where to look for raw data in AnnData.layers
         """
-        if graph_based is None:
-            graph_based = SCANPYDATA_DEFAULT_GRAPH_BASED
-        DataObject.__init__(self, source=source, graph_based=graph_based,
-                            **kwargs)
-        if isinstance(source, AnnData):
-            self.object = source
+        if not isinstance(source, AnnData):
+            source = scanpy.read_h5ad(source, "r")
+        self.object = source
+
+        if "sharing_history" in source.uns:
+            sharing_history = source.uns["sharing_history"].to_dict(orient="records")
+            DataObject.__init__(self, source=source, graph_based=graph_based,
+                                sharing_history=sharing_history,
+                                **kwargs)
         else:
-            self.object = scanpy.read_h5ad(source, "r")
+            DataObject.__init__(self, source=source, graph_based=graph_based,
+                                **kwargs)
         self.raw_key = raw_key
         self.pca_key = pca_key
         if cite_seq_suffix is not None:
@@ -1107,15 +1116,13 @@ class SubclusterData(DataObject):
         return os.path.join(self.study_id, "sub", sub_name)
 
 class SpringData(SubclusterData):
-    def __init__(self, source, graph_based, **kwargs):
+    def __init__(self, source, graph_based = None, **kwargs):
         """Constructor of SpringData object
 
         Keyword arguments:
             source: Path to input file or folder
             graph_based: Name of metadata that is the result of clustering process
         """
-        if graph_based is None:
-            graph_based = SPRINGDATA_DEFAULT_GRAPH_BASED
         DataObject.__init__(self, source=source, graph_based=graph_based,
                             **kwargs)
         self.run_info = self.create_runinfo()
@@ -1240,13 +1247,11 @@ class SpringData(SubclusterData):
         return {"coordinates" : coordinates}
 
 class LoomData(DataObject):
-    def __init__(self, source, graph_based, raw_key="counts",
+    def __init__(self, source, graph_based=None, raw_key="counts",
                     barcode_name=DEFAULT_BARCODE_NAME,
                     feature_name=DEFAULT_FEATURE_NAME,
                     dimred_keys=DEFAULT_DIMRED_KEYS,
                     **kwargs):
-        if graph_based is None:
-            graph_based = LOOMDATA_DEFAULT_GRAPH_BASED
         DataObject.__init__(self, source, graph_based, **kwargs)
         self.raw_key = raw_key
         self.object = loompy.connect(source, "r")
@@ -1323,7 +1328,7 @@ class LoomData(DataObject):
         return dimred
 
 class AbloomData(DataObject):
-    def __init__(self, source, graph_based, raw_key="counts",
+    def __init__(self, source, graph_based=None, raw_key="counts",
                     barcode_name=DEFAULT_ABLOOM_BARCODE_NAME,
                     feature_name=DEFAULT_ABLOOM_FEATURE_NAME,
                     **kwargs):
